//
// Tweak: On-the-fly variable modification for prototyping in processing.
// (c)2010 Teague, Adam Kumpf
// Creative Commons: Share-alike and Attribution
// questions? akumpf@teague.com
//

/*
// Example code...
 
 TweakInt    tHue;
 TweakFloat  tRot;
 TweakDouble tSine;
 TweakBool   tBGisLighter;
 TweakPoint  tPoint;
 TweakString tString;
 
 void setup(){
 size(400, 400);
 colorMode(HSB, 255, 255, 255); 
 frameRate(20);
 
 // setup all our parameters...
 // Tweak types: TweakFloat, TweakInt, TweakPoint, TweakBool, and TweakString.
 // Tweak constructors given as:
 // 1. TweakType(description, value); // TweakString uses this form only.
 // 2. TweakType(description, value, stepSize);
 // 3. TweakType(description, value, stepSize, minValue, maxValue);
 tHue         = new TweakInt   ("Hue", 154);
 tRot         = new TweakFloat ("Rotation Speed", 0.03);
 tSine        = new TweakDouble("Sinewave Multiplier", 0.14);
 tBGisLighter = new TweakBool  ("background is lighter", false);
 tPoint       = new TweakPoint ("Point Offset", new Point(30, -25));
 tString      = new TweakString("Title", "Tweak Example");
 
 // for drawing text
 textFont(createFont("Helvetica", 14));
 }
 
 float lastFullRot = 0;
 void draw(){
 resetMatrix();
 if(tBGisLighter.val){
 background(64, 0, 64);
 }
 else{
 background(0, 0, 0);
 }
 fill(196);
 text(tString.val, 150, 40);
 smooth();
 strokeWeight(2);
 noFill();
 translate(width/2, height/2);
 lastFullRot += tRot.val;
 rotate(lastFullRot);
 pushMatrix();
 stroke(tHue.val, 190, 128);
 float deltaX = (float)(Math.sin(tSine.val*frameCount)*10);
 for(int i=0; i<8; i++){
 ellipse(tPoint.val.x+deltaX, tPoint.val.y, 12, 12);
 rotate(PI/4.0);
 }
 popMatrix();
 }
 */

String TWEAK_VERSION = "10.03.26(b) Alpha";

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

JFrame tweakViewJFrame;
TweakJPanel tweakJPanel;

// tweak mouse handling variables...
Point tweakMouseDownPt;
Point tweakMouseLastPt;
boolean tweakDraggingScroller    = false;
boolean tweakChangingValue       = false;
int tweakYScrollOffset = 0;

// tweak parameter modification variables...
int currentTweakParamIndex       = -1;
TweakParameter currentTweakParam = null;

// tweak list/section variables...
LinkedList tweakParamSectionList = new LinkedList();
LinkedList tweakParamList        = new LinkedList();

// tweak runtime state variables...
boolean tweak_hasBeenInitialized = false;
boolean updateParamView          = true;

void tweak_initParamManagerIfNeeded(){
  if(tweak_hasBeenInitialized){
    requestTweakViewRedraw();
    return;
  }
  println("Tweak: Version " + TWEAK_VERSION);
  tweak_hasBeenInitialized = true;
  tweakViewJFrame = new JFrame("Tweak Panel: " + TWEAK_VERSION);
  tweakViewJFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  tweakViewJFrame.setSize(400, 400);
  tweakViewJFrame.setBackground(new Color(32, 32, 32));
  tweakJPanel = new TweakJPanel();
  tweakViewJFrame.getContentPane().add(tweakJPanel,java.awt.BorderLayout.CENTER);
  tweakViewJFrame.setResizable(false);
  tweakViewJFrame.setVisible(true);
  tweakJPanel.setBackground(new Color(32, 32, 32));
  tweakJPanel.addMouseListener(new MouseListener(){
    public void mousePressed(MouseEvent event){
      tweakMouseDownPt = event.getPoint();
      tweakMouseLastPt = event.getPoint();
      if(tweakMouseDownPt.x > 370){
        tweakDraggingScroller = true;
      }
      else{
        // see which one was clicked...
        int yAbsolute = tweakMouseDownPt.y - tweakYScrollOffset-2;
        int testIndex = yAbsolute/25;
        if(testIndex >= 0 && testIndex < tweakParamList.size()){
          //println("Got index: " + testIndex);
          currentTweakParam = (TweakParameter)tweakParamList.get(testIndex);
          currentTweakParamIndex = testIndex;
          requestTweakViewRedraw();
          tweakChangingValue = true;
        }
      }
    }
    public void mouseReleased(MouseEvent event){
      tweakDraggingScroller = false;
      tweakChangingValue = false;
      currentTweakParamIndex = -1;
      requestTweakViewRedraw();
    }
    public void mouseExited(MouseEvent event){
    }
    public void mouseEntered(MouseEvent event){
    }
    public void mouseClicked(MouseEvent event){
      int yAbsolute = tweakMouseDownPt.y - tweakYScrollOffset-2;
      int testIndex = yAbsolute/25;
      if(testIndex >= 0 && testIndex < tweakParamList.size()){
        //println("Got index: " + testIndex);
        currentTweakParam = (TweakParameter)tweakParamList.get(testIndex);
        if(currentTweakParam.supportsClick()){
          currentTweakParam.handleClick();
        }
        requestTweakViewRedraw();
      }
    }
  }
  );
  tweakJPanel.addMouseMotionListener(new MouseMotionListener(){
    public void mouseDragged(MouseEvent event){
      Point pt = event.getPoint();
      int deltaY = pt.y-tweakMouseLastPt.y;
      int deltaX = pt.x-tweakMouseLastPt.x;
      if(tweakDraggingScroller){
        int maxDeltaY = min(0, -25*(tweakParamList.size()-15));
        tweakYScrollOffset = max(maxDeltaY, min(0, tweakYScrollOffset+deltaY));
        requestTweakViewRedraw();
      }
      if(tweakChangingValue){
        if(currentTweakParam != null){
          if(deltaX > 0){
            for(int i=0; i<deltaX; i++){
              currentTweakParam.incX();
              currentTweakParam.keepInBounds();
              requestTweakViewRedraw();
            }
          }
          if(deltaX < 0){
            for(int i=0; i<-deltaX; i++){
              currentTweakParam.decX();
              currentTweakParam.keepInBounds();
              requestTweakViewRedraw();
            }
          }
          if(deltaY > 0){
            for(int i=0; i<deltaY; i++){
              currentTweakParam.incY();
              currentTweakParam.keepInBounds();
              requestTweakViewRedraw();
            }
          }
          if(deltaY < 0){
            for(int i=0; i<-deltaY; i++){
              currentTweakParam.decY();
              currentTweakParam.keepInBounds();
              requestTweakViewRedraw();
            }
          }
        }
      }
      tweakMouseLastPt = event.getPoint();
    }
    public void mouseMoved(MouseEvent event){
      if(event.getPoint().x > 370){
        tweakViewJFrame.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
      }
      else{
        tweakViewJFrame.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
      }
    }
  }
  );
  tweakJPanel.addMouseWheelListener(new MouseWheelListener(){
    public void mouseWheelMoved(MouseWheelEvent event){
      int maxDeltaY = min(0, -25*(tweakParamList.size()-15));
      tweakYScrollOffset = max(maxDeltaY, min(0, tweakYScrollOffset-event.getUnitsToScroll()*5));
      requestTweakViewRedraw();
    }
  }
  );
  // force an update after a few seconds in case we hit a race condition initially.
  SwingUtilities.invokeLater(new Runnable() {
    public void run() {                     
      delay(3000);
      requestTweakViewRedraw();
    }
  }
  );
}

class TweakJPanel extends JPanel {
  public void paintComponent(Graphics g) {
    // paint params so the user can see what the values are...
    Iterator iter;
    if(updateParamView){
      super.paintComponent(g); // background...
      // draw Scroller...
      noStroke();
      fill(96, 96, 255, 64);
      for(int i=0; i<10; i++){
        g.setColor(new Color(50, 50, 50, 40));
        int yOff = i*i;
        g.fillRect(370, 10+yOff, 30, 360-2*yOff);
      }
      g.setColor(new Color(32, 32, 32, 45));
      g.fillRect(370, 0, 3, 400);
      g.fillRect(370, 0, 2, 400);
      g.fillRect(370, 0, 1, 400);
      g.fillRect(397, 0, 3, 400);
      g.fillRect(398, 0, 2, 400);
      g.fillRect(399, 0, 1, 400);
      // tick marks on the scroller
      for(int i=0; i<13; i++){
        int yTickOff = tweakYScrollOffset%25 + 50 + (int)(i*25);
        int a = 200-min(200, max(0, (int)(abs(yTickOff-190)*1.2))); 
        g.setColor(new Color(128, 128, 128, a));
        g.drawLine(376, yTickOff, 394, yTickOff);
      }
      // translate by how far we've scrolled...
      g.translate(0, tweakYScrollOffset);
      // draw the parameters...
      for(int i=0; i<tweakParamList.size(); i++){
        TweakParameter p = (TweakParameter)tweakParamList.get(i);
        if(i == currentTweakParamIndex){
          g.setColor(Color.white);
        }
        else{
          g.setColor(new Color(160, 160, 160));
        }
        g.drawString(p.desc, 25, 19+i*25);
        g.drawString(p.valString(), 250, 19+i*25);
        // draw +,- sign from orig value.
        int diff = p.equalToOriginalValue();
        g.setColor(new Color(50, 50, 50));
        g.fillOval(220, 8+i*25, 12, 12);
        if(diff > 0){
          g.setColor(new Color(64, 200, 64));
          g.drawString("+", 222, 18+i*25);
        }
        else{
          if(diff < 0){
            g.setColor(new Color(220, 44, 64));
            g.drawString("-", 224, 18+i*25);
          }
        }

        iter = tweakParamSectionList.iterator();
        // check for dividers...
        boolean wasDivided = false;
        while(iter.hasNext()){
          int indx = (Integer)iter.next();
          if(indx == i){
            g.setColor(new Color(128, 128, 128));
            g.drawLine(10, 27+i*25, 360, 27+i*25);
            g.setColor(new Color(16, 16, 16));
            g.drawLine(10, 28+i*25, 360, 28+i*25);
            wasDivided = true;
          }
        }
        if(!wasDivided){
          g.setColor(new Color(40, 40, 40));
          g.drawLine(10, 27+i*25, 360, 27+i*25);
        }
      }
      /*
      // undo translate...
       g.translate(0, -tweakYScrollOffset);
       if(tweakChangingValue && currentTweakParam != null){
       // draw a line from the start point...
       g.setColor(new Color(128, 128, 128));
       if(currentTweakParam.supportsX()){
       if(currentTweakParam.supportsY()){
       // X and Y
       g.drawLine(tweakMouseDownPt.x, tweakMouseDownPt.y,  tweakMouseLastPt.x, tweakMouseLastPt.y);
       }else{
       // X only
       g.drawLine(tweakMouseDownPt.x, tweakMouseDownPt.y, tweakMouseLastPt.x, tweakMouseDownPt.y);
       }
       }else{
       if(currentTweakParam.supportsY()){
       // Y only
       g.drawLine(tweakMouseDownPt.x, tweakMouseDownPt.y, tweakMouseDownPt.x, tweakMouseLastPt.y);
       }
       }
       }
       */
      updateParamView = false;
    }
  }
}

void requestTweakViewRedraw(){
  updateParamView = true;
  if(tweakViewJFrame != null){
    tweakViewJFrame.repaint();
  }
}


// -------------------------------------------------------------------------------------------------------
// #######################################################################################################
// -------------------------------------------------------------------------------------------------------

abstract class TweakParameter{
  String desc = "?";
  abstract void keepInBounds();
  abstract void incX();
  abstract void decX();
  abstract void incY();
  abstract void decY();
  abstract void handleClick();
  abstract boolean supportsX();
  abstract boolean supportsY();
  abstract boolean supportsClick();
  String toString(){
    return "TweakParameter: [" + desc + " @ " + valString() + "]";
  }
  // -1 = less than, 0 = equal, +1 = greater than original/default value.
  abstract int equalToOriginalValue(); 
  abstract String valString();
}

void tweakAddSectionDivider(){
  if(tweakParamList.size() > 0){
    tweakParamSectionList.add(tweakParamList.size()-1);
  }
  else{
    println("could not add section divider before any entries...");
  }
}

class TweakFloat extends TweakParameter{
  float origVal;
  float val;
  float maxVal;
  float minVal;
  float step;
  public TweakFloat(String desc, float val, float step, float minVal, float maxVal){
    this.val     = val;
    this.origVal = val;
    this.minVal  = minVal;
    this.maxVal  = maxVal;
    this.step    = step;
    this.desc    = desc;
    tweakParamList.add(this);
    tweak_initParamManagerIfNeeded();
  }
  public TweakFloat(String desc, float val, float step){
    this(desc, val, step, -Float.MAX_VALUE, Float.MAX_VALUE);
  }
  public TweakFloat(String desc, float val){
    this(desc, val, 0.01);
  }
  boolean supportsX(){
    return true;
  }
  boolean supportsY(){
    return false;
  }
  boolean supportsClick(){
    return false;
  }
  void incX(){
    val += step;
  }
  void decX(){
    val -= step;
  }
  void incY(){
  }
  void decY(){
  }
  void handleClick(){
  }
  int equalToOriginalValue(){
    if(val < origVal){
      return -1;
    }
    else{
      if(val > origVal){
        return +1;
      }
      else{
        return 0;
      }
    }
  }
  String valString(){
    return ""+String.format("%f", val);
  }
  void keepInBounds(){
    if(val > maxVal){
      val = maxVal;
    }
    if(val < minVal){
      val = minVal;
    }
  }
}

class TweakInt extends TweakParameter{
  int origVal;
  int val;
  int maxVal;
  int minVal;
  int step;
  public TweakInt(String desc, int val, int step, int minVal, int maxVal){
    this.val     = val;
    this.origVal = val;
    this.minVal  = minVal;
    this.maxVal  = maxVal;
    this.step    = step;
    this.desc    = desc;
    tweakParamList.add(this);
    tweak_initParamManagerIfNeeded();
  }
  public TweakInt(String desc, int val, int step){
    this(desc, val, step, -Integer.MAX_VALUE, Integer.MAX_VALUE);
  }
  public TweakInt(String desc, int val){
    this(desc, val, 1);
  }
  boolean supportsX(){
    return true;
  }
  boolean supportsY(){
    return false;
  }
  boolean supportsClick(){
    return false;
  }
  void incX(){
    val += step;
  }
  void decX(){
    val -= step;
  }
  void incY(){
  }
  void decY(){
  }
  void handleClick(){
  }
  int equalToOriginalValue(){
    if(val < origVal){
      return -1;
    }
    else{
      if(val > origVal){
        return +1;
      }
      else{
        return 0;
      }
    }
  }
  String valString(){
    return ""+val;
  }
  void keepInBounds(){
    if(val > maxVal){
      val = maxVal;
    }
    if(val < minVal){
      val = minVal;
    }
  }
}

class TweakDouble extends TweakParameter{
  double origVal;
  double val;
  double maxVal;
  double minVal;
  double step;
  public TweakDouble(String desc, double val, double step, double minVal, double maxVal){
    this.val     = val;
    this.origVal = val;
    this.minVal  = minVal;
    this.maxVal  = maxVal;
    this.step    = step;
    this.desc    = desc;
    tweakParamList.add(this);
    tweak_initParamManagerIfNeeded();
  }
  public TweakDouble(String desc, double val, double step){
    this(desc, val, step, -Double.MAX_VALUE, Double.MAX_VALUE);
  }
  public TweakDouble(String desc, double val){
    this(desc, val, 0.01);
  }
  boolean supportsX(){
    return true;
  }
  boolean supportsY(){
    return false;
  }
  boolean supportsClick(){
    return false;
  }
  void incX(){
    val += step;
  }
  void decX(){
    val -= step;
  }
  void incY(){
  }
  void decY(){
  }
  void handleClick(){
  }
  int equalToOriginalValue(){
    if(val < origVal){
      return -1;
    }
    else{
      if(val > origVal){
        return +1;
      }
      else{
        return 0;
      }
    }
  }
  String valString(){
    return  ""+String.format("%f", val);
  }
  void keepInBounds(){
    if(val > maxVal){
      val = maxVal;
    }
    if(val < minVal){
      val = minVal;
    }
  }
}

class TweakPoint extends TweakParameter{
  Point origVal;
  Point val;
  Point maxVal;
  Point minVal;
  Point step;
  public TweakPoint(String desc, Point val, Point step, Point minVal, Point maxVal){
    this.val     = val;
    this.origVal = new Point(val);
    this.minVal  = minVal;
    this.maxVal  = maxVal;
    this.step    = step;
    this.desc    = desc;
    tweakParamList.add(this);
    tweak_initParamManagerIfNeeded();
  }
  public TweakPoint(String desc, Point val, Point step){
    this(desc, val, step, new Point(-Integer.MAX_VALUE, -Integer.MAX_VALUE),  new Point(Integer.MAX_VALUE, Integer.MAX_VALUE));
  }
  public TweakPoint(String desc, Point val){
    this(desc, val, new Point(1,1));
  }
  boolean supportsX(){
    return true;
  }
  boolean supportsY(){
    return true;
  }
  boolean supportsClick(){
    return false;
  }
  void incX(){
    val.x += step.x;
  }
  void decX(){
    val.x -= step.x;
  }
  void incY(){
    val.y += step.y;
  }
  void decY(){
    val.y -= step.y;
  }
  void handleClick(){
  }
  int equalToOriginalValue(){
    if(val.x == origVal.x && val.y == origVal.y){
      return 0;
    }
    else{
      return 1;
    }
  }
  String valString(){
    return "("+val.x+","+val.y+")";
  }
  void keepInBounds(){
    if(val.x > maxVal.x){
      val.x = maxVal.x;
    }
    if(val.x < minVal.x){
      val.x = minVal.x;
    }
    if(val.y > maxVal.y){
      val.y = maxVal.y;
    }
    if(val.y < minVal.y){
      val.y = minVal.y;
    }
  }
}


class TweakBool extends TweakParameter{
  boolean origVal;
  boolean val;
  public TweakBool(String desc, boolean val, boolean step, boolean minVal, boolean maxVal){
    this.val     = val;
    this.origVal = val;
    this.desc    = desc;
    tweakParamList.add(this);
    tweak_initParamManagerIfNeeded();
  }
  public TweakBool(String desc, boolean val, boolean step){
    this(desc, val, step, false, true);
  }
  public TweakBool(String desc, boolean val){
    this(desc, val, true);
  }
  boolean supportsX(){
    return true;
  }
  boolean supportsY(){
    return false;
  }
  boolean supportsClick(){
    return false;
  }
  void incX(){
    val = true;
  }
  void decX(){
    val = false;
  }
  void incY(){
  }
  void decY(){
  }
  void handleClick(){
  }
  int equalToOriginalValue(){
    if(origVal == true){
      if(val == false){
        return -1;
      }
      else{
        return 0;
      }
    }
    else{
      if(val == false){
        return 0;
      }
      else{
        return 1;
      }
    }
  }
  String valString(){
    return ""+val;
  }
  void keepInBounds(){
  }
}


class TweakString extends TweakParameter{
  String origVal;
  String val;
  public TweakString(String desc, String val){
    this.val     = val;
    this.origVal = val;
    this.desc    = desc;
    tweakParamList.add(this);
    tweak_initParamManagerIfNeeded();
  }
  boolean supportsX(){
    return false;
  }
  boolean supportsY(){
    return false;
  }
  boolean supportsClick(){
    return true;
  }
  void incX(){
  }
  void decX(){
  }
  void incY(){
  }
  void decY(){
  }
  void handleClick(){
    SwingUtilities.invokeLater(new Runnable() { //  IMPORTANT FOR SWIN COMPONENTS TO NOT KILL THINGS
      public void run() {                       //  ----- (put them inside an "invokeLater" method like this)...
        String resultText = (String)JOptionPane.showInputDialog(null, 
        "Change the value of: "+desc+"          \n", 
        "Tweak: " + desc, 
        JOptionPane.QUESTION_MESSAGE, null, null, val);
        if(resultText != null){
          //println("Updating String: " + resultText);
          val = resultText;
          requestTweakViewRedraw();
        }
      }
    }
    );
  }
  int equalToOriginalValue(){
    if(origVal.equals(val)){
      return 0;
    }
    else{
      return 1;
    }
  }
  String valString(){
    if(val.length() > 11){
      return val.substring(0,9)+"...";
    }
    else{
      return ""+val;
    }
  }
  void keepInBounds(){
  }
}














