//
// A more complex dynamic design using Tweak.
// (c)2010 Teague, Adam Kumpf
// Creative Commons: Share-alike and Attribution
// questions? akumpf@teague.com
//


TweakInt    tHue;
TweakInt    tSaturation;
TweakInt    tBrightness;
TweakInt    tNumLines;
TweakString tString;
TweakFloat  tRotA;
TweakFloat  tCurlyRes;
TweakInt    tNumCurlySegs;
TweakFloat  tDriftMul;
TweakFloat  tRandMul;
TweakFloat  tBreathMul;
TweakFloat  tBreathSpeed;
TweakFloat  tRotB;
TweakFloat  tLineUniqueness;
TweakFloat  tRotAMul;
TweakPoint  tPointC;
TweakFloat  tRotAboutC;
TweakBool   tBGisLighter;

void setup(){
  size(600, 600);

  // setup all our parameters...
  // Tweak types: TweakFloat, TweakInt, TweakPoint, and TweakBool.
  // Tweak constructors given as:
  // 1. TweakType(description, value);
  // 2. TweakType(description, value, stepSize);
  // 3. TweakType(description, value, stepSize, minValue, maxValue);
  tHue            = new TweakInt("hue", 176, 1, 0, 255);
  tSaturation     = new TweakInt("saturation", 198, 1, 0, 255);
  tBrightness     = new TweakInt("brightness", 228, 1, 0, 255);
  tBGisLighter    = new TweakBool("background is lighter", false);
  tweakAddSectionDivider();
  tString         = new TweakString("moving text", "@-'-,---");
  tRotA           = new TweakFloat("rotation A", -PI/20);
  tRotAMul        = new TweakFloat("rotation A breath", 0.09, PI/1000, -PI/20, PI/20);
  tRotB           = new TweakFloat("rotation B", 0.035);
  tweakAddSectionDivider();
  tNumLines       = new TweakInt("total lines", 6, 1, 1, 20);
  tCurlyRes       = new TweakFloat("curly resolution", 8.0, 0.5, 1.0, 30.0);
  tNumCurlySegs   = new TweakInt("curly segments", 80, 1, 1, 100);
  tDriftMul       = new TweakFloat("curly drift", 2.4, .1, 0, 10);
  tRandMul        = new TweakFloat("random noise", 0.000, 0.0005, 0, .2);
  tBreathMul      = new TweakFloat("breathing", 15, .1, 0, 100);
  tBreathSpeed    = new TweakFloat("breath speed", 0.015, 0.005, 0, 0.1);
  tLineUniqueness = new TweakFloat("line uniqueness", 0, 0.01, 0, 1);
  tweakAddSectionDivider();
  tPointC         = new TweakPoint("point C", new Point(0, 0));
  tRotAboutC      = new TweakFloat("rotation about C", 0, PI/2000, -PI/20, PI/20);
  colorMode(HSB, 255, 255, 255); 
  
  // for drawing text
  textFont(createFont("Helvetica", 14));
}

float lastFullRot = 0;
float lastFullCRot = 0;

void draw(){
  resetMatrix();
  if(tBGisLighter.val){
    background(64, 0, 64);
  }
  else{
    background(0, 0, 0);
  }
  smooth();
  strokeWeight(2);
  stroke(tHue.val, tSaturation.val, tBrightness.val);
  float rot = TWO_PI / (float)tNumLines.val;
  translate(width/2, height/2);
  lastFullCRot += tRotAboutC.val;
  rotate(lastFullCRot);
  translate(tPointC.val.x, tPointC.val.y);
  float deltaRot = 0.1*tRotB.val;
  rotate(lastFullRot+deltaRot);
  lastFullRot += deltaRot;
  for(int i=0; i<tNumLines.val; i++){
    pushMatrix();
    {
      // move to center and rotate.
      rotate(rot*i);
      pushMatrix();
      {
        // draw curly stroke
        for(int c=0; c<tNumCurlySegs.val; c++){
          stroke(tHue.val, tSaturation.val, tBrightness.val*((float)Math.sin(c/10.0)+1.0)/2.0);
          float y = tBreathMul.val*(float)Math.sin(frameCount*tBreathSpeed.val);
          float x = tCurlyRes.val+10.0*(float)Math.sin(tDriftMul.val*(float)c/10.0+i*27.2*tLineUniqueness.val);
          line(0, 0, x, y);
          translate(x,  y);
          rotate(tRotA.val+tRandMul.val*random(1.0) + tRotAMul.val*(float)Math.sin(frameCount*tBreathSpeed.val/PI));
        }
        // draw text
        fill(tHue.val, tSaturation.val, tBrightness.val);
        text(tString.val, 50, 50);
      }
      popMatrix();
    }
    popMatrix();
  }
}







/*
TweakInt    tHue;
 TweakFloat  tRot;
 TweakDouble tSine;
 TweakPoint  tPoint;
 
 
 void setup(){
 size(400, 400);
 colorMode(HSB, 255, 255, 255); 
 frameRate(20);
 
 // setup all our parameters...
 // Tweak constructors given as:
 // 1. TweakType(description, value);
 // 2. TweakType(description, value, stepSize);
 // 3. TweakType(description, value, stepSize, minValue, maxValue);
 //tweak_hasBeenInitialized = true;
 tHue   = new TweakInt   ("Hue", 154);
 tRot   = new TweakFloat ("Rotation Speek", 0.03);
 tSine  = new TweakDouble("Sinewave Multiplier", 0.14);
 tPoint = new TweakPoint ("Point Offset", new Point(30, -25));
 }
 
 float lastFullRot = 0;
 void draw(){
 resetMatrix();
 background(0, 0, 0);
 smooth();
 strokeWeight(2);
 noFill();
 translate(width/2, height/2);
 lastFullRot += tRot.val;
 rotate(lastFullRot);
 pushMatrix();
 stroke(tHue.val, 190, 128);
 float deltaX = (float)(Math.sin(tSine.val*frameCount)*10);
 for(int i=0; i<8; i++){
 ellipse(tPoint.val.x+deltaX, tPoint.val.y, 12, 12);
 rotate(PI/4.0);
 }
 popMatrix();
 }
 
 */





